package com.rjconsultores.ventaboletos.dao.hibernate;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import com.rjconsultores.ventaboletos.dao.ConvenioTransportadoraDAO;
import com.rjconsultores.ventaboletos.entidad.ConvenioTransportadora;

@Repository("convenioTransportadoraDAO")
public class ConvenioTransportadoraHibernateDAO extends GenericHibernateDAO<ConvenioTransportadora, Long> implements ConvenioTransportadoraDAO {

	@Autowired
    public ConvenioTransportadoraHibernateDAO(@Qualifier("sessionFactory") SessionFactory factory) {
        setSessionFactory(factory);
    }

	@Override
    public List<ConvenioTransportadora> obtenerTodos() {
        Criteria c = getSession().createCriteria(getPersistentClass());
        c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));

        return castList(ConvenioTransportadora.class, c.list())  ;
    }

	@Override
	public ConvenioTransportadora buscarPelaTransportadoraId(Long transportadoraId) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("transportadora.transportadoraId", transportadoraId));
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));

		return (ConvenioTransportadora)c.uniqueResult();
	}

	@Override
	public boolean existe(ConvenioTransportadora convenio) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("transportadora", convenio.getTransportadora()));
		c.add(Restrictions.eq("numConvenio", convenio.getNumConvenio()));
		c.add(Restrictions.eq("dataInicial", convenio.getDataInicial()));
		c.add(Restrictions.eq("dataFinal", convenio.getDataFinal()));
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));

		return c.uniqueResult()!=null;
	}

}
