package com.rjconsultores.ventaboletos.entidad;

import java.math.BigDecimal;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import com.rjconsultores.ventaboletos.anotacao.Renderizado;
import com.rjconsultores.ventaboletos.enums.TipoLancamentoCaixa;

import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

@Getter
@Setter
@EqualsAndHashCode
@NoArgsConstructor
@Entity
@Table(name = "CAIXA_CONTRATO")
@SequenceGenerator(name = "CAIXA_CONTRATO_SEQ", sequenceName = "CAIXA_CONTRATO_SEQ", allocationSize = 1)
public class CaixaContrato {

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "CAIXA_CONTRATO_SEQ")
    @Column(name = "CAIXACONTRATO_ID")
    private Long id;

    @Column(name = "CONTRATO_ID")
    private Long contratoId;

    @Column(name = "DATA_LANCAMENTO")
    @Temporal(TemporalType.TIMESTAMP)
    @Renderizado
    private Date dataLancamento;
    
    @Renderizado
    @Column(name = "VALOR")
    private BigDecimal valor;
    
    @Column(name = "OBSERVACAO")
    private String observacao;    

    @Renderizado( conversor = TipoLancamentoCaixa.class )
    @Column(name = "TIPO_LANCAMENTO_ID")
    private Integer tipoLancamentoId;

    @Column(name = "CAJA_ID")
    private Long cajaId;

    @Column(name = "EVENTOEXTRA_ID")
    private Long eventoExtraId;

    @Column(name = "USUARIO_ID")
    private Integer usuarioId;

    @Column(name = "ACTIVO")
    private boolean activo;

    @Column(name = "FECMODIF")
    @Temporal(TemporalType.TIMESTAMP)
    private Date fecModif;
    
    public TipoLancamentoCaixa getTipoLancamentoCaixa() {
    	return TipoLancamentoCaixa.getTipoFormapagoByValor(tipoLancamentoId);
    }

}