package com.rjconsultores.ventaboletos.entidad;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.Hibernate;

import br.com.rjconsultores.auditador.annotations.AuditarClasse;
import br.com.rjconsultores.auditador.annotations.NaoAuditar;
import br.com.rjconsultores.auditador.interfaces.Auditavel;

/**
 * @author vjcor
 *
 */
@AuditarClasse(nome = "CobrancaAdcPuntoVenta", tela = "auditarClasse.CobrancaAdcPuntoVenta")
@Entity
@SequenceGenerator(name = "COBRANCA_ADC_SEQ", sequenceName = "COBRANCA_ADC_SEQ", allocationSize = 1)
@Table(name = "COBRANCA_ADC_PUNTO_VENTA")
public class CobrancaAdcPuntoVenta implements Serializable, Auditavel<CobrancaAdcPuntoVenta> {

	private static final long serialVersionUID = 1L;
	
	@Id
	@Basic(optional = false)
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "COBRANCA_ADC_SEQ")
	@Column(name = "COBRANCAADDPUNTOVENTA_ID")
	private Integer cobrancaAddPuntoVenta_id;
	
	@JoinColumn(name = "PUNTOVENTA_ID")
	@ManyToOne
	private PuntoVenta puntoVenta;
	
	@Column(name = "DESCRICAO")
	private String descricao;
	
	@Column(name = "VALOR")
	private BigDecimal valor;
	
	@OneToOne
	@JoinColumn(name = "EMPRESA_ID")
	private Empresa empresa;
	
	@Column(name = "DIARIO")
	private Boolean diario;
	
	@Column(name = "SEMANAL")
	private Boolean semanal;
	
	@Column(name = "MENSAL")
	private Boolean mensal;
	
	@Column(name = "SEGUNDA")
	private Boolean segunda;
	
	@Column(name = "TERCA")
	private Boolean terca;
	
	@Column(name = "QUARTA")
	private Boolean quarta;
	
	@Column(name = "QUINTA")
	private Boolean quinta;
	
	@Column(name = "SEXTA")
	private Boolean sexta;
	
	@Column(name = "PRIMEIRO_DIA_UTIL")
	private Boolean primeiroDiaUtil;
	
	@Column(name = "ULTIMO_DIA_UTIL")
	private Boolean ultimoDiaUtil;
	
	@Column(name = "DESCONTO_PROXIMO_CICLO")
	private Boolean descontarProximoCiclo;
	
	@Column(name = "DESCONTO_PROXIMO_DIA")
	private Boolean descontarProximoDia;
	
	@Column(name = "DATAINICIAL")
	@Temporal(TemporalType.TIMESTAMP)
	private Date dataInicial;
	
	@Column(name = "DATAFINAL")
	@Temporal(TemporalType.TIMESTAMP)
	private Date dataFinal;
	
	@Column(name = "ACTIVO")
	private Boolean activo;
	
	@Column(name = "FECMODIF")
	@Temporal(TemporalType.TIMESTAMP)
	private Date fecmodif;
	
	@Column(name = "USUARIO_ID")
	private Integer usuarioId;
	
	@Transient
	@NaoAuditar
	private CobrancaAdcPuntoVenta cobrancaAdcPuntoVentaClone;
	
	private Boolean getBoolean(Boolean valor) {
		return valor == null ? false : valor;
	}
	
	public Integer getUsuarioId() {
		return usuarioId;
	}


	public void setUsuarioId(Integer usuarioId) {
		this.usuarioId = usuarioId;
	}


	public Integer getCobrancaAddPuntoVenta_id() {
		return cobrancaAddPuntoVenta_id;
	}

	public void setCobrancaAddPuntoVenta_id(Integer cobrancaAddPuntoVenta_id) {
		this.cobrancaAddPuntoVenta_id = cobrancaAddPuntoVenta_id;
	}

	public PuntoVenta getPuntoVenta() {
		return puntoVenta;
	}

	public void setPuntoVenta(PuntoVenta puntoVenta) {
		this.puntoVenta = puntoVenta;
	}

	public String getDescricao() {
		return descricao;
	}

	public void setDescricao(String descricao) {
		this.descricao = descricao;
	}

	public BigDecimal getValor() {
		return valor;
	}

	public void setValor(BigDecimal valor) {
		this.valor = valor;
	}

	public Empresa getEmpresa() {
		return empresa;
	}

	public void setEmpresa(Empresa empresa) {
		this.empresa = empresa;
	}

	public Boolean getDiario() {
		return getBoolean(diario);
	}

	public void setDiario(Boolean diario) {
		this.diario = diario;
	}

	public Boolean getSemanal() {
		return getBoolean(semanal);
	}

	public void setSemanal(Boolean semanal) {
		this.semanal = semanal;
	}

	public Boolean getMensal() {
		return getBoolean(mensal);
	}

	public void setMensal(Boolean mensal) {
		this.mensal = mensal;
	}

	public Boolean getSegunda() {
		return getBoolean(segunda);
	}

	public void setSegunda(Boolean segunda) {
		this.segunda = segunda;
	}

	public Boolean getTerca() {
		return getBoolean(terca);
	}

	public void setTerca(Boolean terca) {
		this.terca = terca;
	}

	public Boolean getQuarta() {
		return getBoolean(quarta);
	}

	public void setQuarta(Boolean quarta) {
		this.quarta = quarta;
	}

	public Boolean getQuinta() {
		return getBoolean(quinta);
	}

	public void setQuinta(Boolean quinta) {
		this.quinta = quinta;
	}

	public Boolean getSexta() {
		return getBoolean(sexta);
	}

	public void setSexta(Boolean sexta) {
		this.sexta = sexta;
	}

	public Boolean getPrimeiroDiaUtil() {
		return getBoolean(primeiroDiaUtil);
	}

	public void setPrimeiroDiaUtil(Boolean primeiroDiaUtil) {
		this.primeiroDiaUtil = primeiroDiaUtil;
	}

	public Boolean getUltimoDiaUtil() {
		return getBoolean(ultimoDiaUtil);
	}

	public void setUltimoDiaUtil(Boolean ultimoDiaUtil) {
		this.ultimoDiaUtil = ultimoDiaUtil;
	}

	public Boolean getActivo() {
		return getBoolean(activo);
	}

	public void setActivo(Boolean activo) {
		this.activo = activo;
	}

	public Date getFecmodif() {
		return fecmodif;
	}

	public void setFecmodif(Date fecmodif) {
		this.fecmodif = fecmodif;
	}

	public Boolean getDescontarProximoCiclo() {
		return descontarProximoCiclo;
	}

	public void setDescontarProximoCiclo(Boolean descontarProximoCiclo) {
		this.descontarProximoCiclo = descontarProximoCiclo;
	}

	public Boolean getDescontarProximoDia() {
		return descontarProximoDia;
	}

	public void setDescontarProximoDia(Boolean descontarProximoDia) {
		this.descontarProximoDia = descontarProximoDia;
	}

	public Date getDataInicial() {
		return dataInicial;
	}

	public void setDataInicial(Date dataInicial) {
		this.dataInicial = dataInicial;
	}

	public Date getDataFinal() {
		return dataFinal;
	}

	public void setDataFinal(Date dataFinal) {
		this.dataFinal = dataFinal;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((cobrancaAddPuntoVenta_id == null) ? 0 : cobrancaAddPuntoVenta_id.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CobrancaAdcPuntoVenta other = (CobrancaAdcPuntoVenta) obj;
		if (cobrancaAddPuntoVenta_id == null) {
			if (other.cobrancaAddPuntoVenta_id != null)
				return false;
		} else if (!cobrancaAddPuntoVenta_id.equals(other.cobrancaAddPuntoVenta_id))
			return false;
		return true;
	}
	
	@Override
	public String toString() {
		return this.getDescricao();
	}

	@Override
	public void clonar() throws CloneNotSupportedException {
		
		cobrancaAdcPuntoVentaClone = new CobrancaAdcPuntoVenta();
		cobrancaAdcPuntoVentaClone = (CobrancaAdcPuntoVenta) this.clone();
		Hibernate.initialize(cobrancaAdcPuntoVentaClone.getPuntoVenta());
		
	}

	@Override
	public CobrancaAdcPuntoVenta getCloneObject() throws CloneNotSupportedException {
		return cobrancaAdcPuntoVentaClone;
	}

	@Override
	public String getTextoInclusaoExclusao() {
		return String.format("ID [%s]", getCobrancaAddPuntoVenta_id());
	}
}
