package com.rjconsultores.ventaboletos.entidad;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import br.com.rjconsultores.auditador.annotations.AuditarClasse;
import br.com.rjconsultores.auditador.annotations.NaoAuditar;
import br.com.rjconsultores.auditador.interfaces.Auditavel;

@Entity
@Table(name = "FECHAMENTO_PARAMPTOVTA")
public class FechamentoParamptovta implements Serializable {

	private static final long serialVersionUID = 1L;

	private Long fechamentoparamptovtaId;
	private PuntoVenta puntoventa;
	private Empresa empresa;
	private Integer intervalofechamento;
	private Date fecmodif;
	private Integer usuarioId;
	private Boolean activo;
	private Integer diasemtransito;
	private String tipopagamento;


	public FechamentoParamptovta() {
	}

	public FechamentoParamptovta( PuntoVenta puntoventa, Empresa empresa, Integer usuarioId) {		
		this.puntoventa = puntoventa;
		this.empresa = empresa;
		
		this.fecmodif = new Date();
		this.usuarioId = usuarioId;
		this.activo = true;
	}

	public FechamentoParamptovta(
			PuntoVenta puntoventa, Empresa empresa, Integer intervalofechamento,
			Date fecmodif, Integer usuarioId, Boolean activo) {		
		this.puntoventa = puntoventa;
		this.empresa = empresa;
		this.intervalofechamento = intervalofechamento;
		this.fecmodif = fecmodif;
		this.usuarioId = usuarioId;
		this.activo = activo;
	}

	
	@SequenceGenerator(name = "FECHAMENTO_PARAMPTOVTA_SEQ", sequenceName = "FECHAMENTO_PARAMPTOVTA_SEQ", allocationSize = 1)
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "FECHAMENTO_PARAMPTOVTA_SEQ")
	@Column(name = "FECHAMENTOPARAMPTOVTA_ID", unique = true, nullable = false, precision = 15, scale = 0)
	public Long getFechamentoparamptovtaId() {
		return this.fechamentoparamptovtaId;
	}

	public void setFechamentoparamptovtaId(Long fechamentoparamptovtaId) {
		this.fechamentoparamptovtaId = fechamentoparamptovtaId;
	}

	public void setIntervalofechamento(Integer intervalofechamento) {
		this.intervalofechamento = intervalofechamento;
	}

	@Temporal(TemporalType.TIMESTAMP)
	@Column(name = "FECMODIF", length = 7)
	public Date getFecmodif() {
		return fecmodif;
	}
	public void setFecmodif(Date fecmodif) {
		this.fecmodif = fecmodif;
	}
	
	@Column(name = "USUARIO_ID", precision = 7, scale = 0)
	public Integer getUsuarioId() {
		return usuarioId;
	}
	public void setUsuarioId(Integer usuarioId) {
		this.usuarioId = usuarioId;
	}
	
	@Column(name = "ACTIVO", precision = 1, scale = 0)
	public Boolean getActivo() {
		return activo;
	}
	public void setActivo(Boolean activo) {
		this.activo = activo;
	}

	@JoinColumn(name = "PUNTOVENTA_ID")
	@ManyToOne
	public PuntoVenta getPuntoventa() {
		return puntoventa;
	}

	public void setPuntoventa(PuntoVenta puntoventa) {
		this.puntoventa = puntoventa;
	}

	@JoinColumn(name = "EMPRESA_ID")
	@ManyToOne
	public Empresa getEmpresa() {
		return empresa;
	}

	public void setEmpresa(Empresa empresa) {
		this.empresa = empresa;
	}

	@Column(name = "INTERVALOFECHAMENTO", length = 2)
	public Integer getIntervalofechamento() {
		return intervalofechamento;
	}

	@Column(name = "DIASEMTRANSITO", length = 2)
	public Integer getDiasemtransito() {
		return diasemtransito;
	}

	public void setDiasemtransito(Integer diasemtransito) {
		this.diasemtransito = diasemtransito;
	}

	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("FechamentoParamptovta [fechamentoparamptovtaId=").append(fechamentoparamptovtaId).append(", puntoventa=").append(puntoventa).append(", empresa=").append(empresa).append(", intervalofechamento=").append(intervalofechamento).append(", fecmodif=").append(fecmodif).append(", usuarioId=").append(usuarioId).append(", activo=").append(activo).append(", diasemtransito=").append(diasemtransito).append("]");
		return builder.toString();
	}

	@Column(name = "TIPOPAGAMENTO", length = 1)
	public String getTipopagamento() {
		return tipopagamento;
	}

	public void setTipopagamento(String tipopagamento) {
		this.tipopagamento = tipopagamento;
	}
		
}
