package com.rjconsultores.ventaboletos.entidad;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.Hibernate;

import br.com.rjconsultores.auditador.annotations.AuditarClasse;
import br.com.rjconsultores.auditador.annotations.NaoAuditar;
import br.com.rjconsultores.auditador.interfaces.Auditavel;

/**
 * @author Valdevir
 */
@AuditarClasse(nome = "PtovtaExcecaoMultaCanc", tela = "auditarClasse.PtovtaExcecaoMultaCanc")
@Entity
@SequenceGenerator(name = "PTOVTA_EXCECAO_MULTA_CANC_SEQ", sequenceName = "PTOVTA_EXCECAO_MULTA_CANC_SEQ", allocationSize = 1)
@Table(name = "PTOVTA_EXCECAO_MULTA_CANC")
public class PtovtaExcecaoMultaCanc implements Serializable , Auditavel<PtovtaExcecaoMultaCanc> {
	
    private static final long serialVersionUID = 1L;
    
    @Id
    @Basic(optional = false)
    @GeneratedValue(strategy = GenerationType.AUTO, generator = "PTOVTA_EXCECAO_MULTA_CANC_SEQ")
    @Column(name = "PTOVTAEXCECAOMULTACANC_ID")
    private Integer ptovtaExcecaoMultaCancId;
    
    @ManyToOne
    @JoinColumn(name = "PUNTOVENTA_ID")
    private PuntoVenta puntoventa;
    
    @ManyToOne
    @JoinColumn(name = "EMPRESA_ID")    
    private Empresa empresa;
    
    @ManyToOne
    @JoinColumn(name = "ORGAOCONCEDENTE_ID")    
    private OrgaoConcedente orgaoConcedente;
        
    @Column(name = "ACTIVO")
    private Boolean activo;
    
    @Column(name = "FECMODIF")
    @Temporal(TemporalType.TIMESTAMP)
    private Date fecmodif;
    
    @Column(name = "USUARIO_ID")
    private int usuarioId;
    
	@Transient
	@NaoAuditar
	private PtovtaExcecaoMultaCanc ptovtaExcecaoMultaCancClone;
   
    public PtovtaExcecaoMultaCanc() {
    	super();
    	this.activo = true;
    	this.fecmodif = new Date();
    }
       
	public Integer getPtovtaExcecaoMultaCancId() {
		return ptovtaExcecaoMultaCancId;
	}

	public void setPtovtaExcecaoMultaCancId(Integer ptovtaExcecaoMultaCancId) {
		this.ptovtaExcecaoMultaCancId = ptovtaExcecaoMultaCancId;
	}

	public OrgaoConcedente getOrgaoConcedente() {
		return orgaoConcedente;
	}

	public void setOrgaoConcedente(OrgaoConcedente orgaoConcedente) {
		this.orgaoConcedente = orgaoConcedente;
	}

	public PtovtaExcecaoMultaCanc getPtovtaExcecaoMultaCancClone() {
		return ptovtaExcecaoMultaCancClone;
	}

	public void setPtovtaExcecaoMultaCancClone(PtovtaExcecaoMultaCanc ptovtaExcecaoMultaCancClone) {
		this.ptovtaExcecaoMultaCancClone = ptovtaExcecaoMultaCancClone;
	}

	public PuntoVenta getPuntoventa() {
		return puntoventa;
	}

	public void setPuntoventa(PuntoVenta puntoventa) {
		this.puntoventa = puntoventa;
	}

	public Empresa getEmpresa() {
		return empresa;
	}

	public void setEmpresa(Empresa empresa) {
		this.empresa = empresa;
	}


	public Boolean getActivo() {
		return activo;
	}

	public void setActivo(Boolean activo) {
		this.activo = activo;
	}

	public Date getFecmodif() {
		return fecmodif;
	}

	public void setFecmodif(Date fecmodif) {
		this.fecmodif = fecmodif;
	}

	public int getUsuarioId() {
		return usuarioId;
	}

	public void setUsuarioId(int usuarioId) {
		this.usuarioId = usuarioId;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((ptovtaExcecaoMultaCancId == null) ? 0 : ptovtaExcecaoMultaCancId.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof PtovtaExcecaoMultaCanc))
			return false;
		PtovtaExcecaoMultaCanc other = (PtovtaExcecaoMultaCanc) obj;
		if (ptovtaExcecaoMultaCancId == null) {
			if (other.ptovtaExcecaoMultaCancId != null)
				return false;
		} else if (!ptovtaExcecaoMultaCancId.equals(other.ptovtaExcecaoMultaCancId))
			return false;
		return true;
	}
	
	@Override
	public void clonar() throws CloneNotSupportedException {
		ptovtaExcecaoMultaCancClone = new PtovtaExcecaoMultaCanc();
		ptovtaExcecaoMultaCancClone = (PtovtaExcecaoMultaCanc) this.clone();
		Hibernate.initialize(ptovtaExcecaoMultaCancClone.getPuntoventa());
		
	}

	@Override
	public PtovtaExcecaoMultaCanc getCloneObject() throws CloneNotSupportedException {
		return ptovtaExcecaoMultaCancClone;
	}

	@Override
	public String getTextoInclusaoExclusao() {
		return String.format("ID [%s]", getPtovtaExcecaoMultaCancId());
	}

    
}
