package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.ComissaoDAO;
import com.rjconsultores.ventaboletos.entidad.Comissao;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.service.ComissaoService;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.vo.comissao.HistoricoComissao;

@Service("comissaoService")
public class ComissaoServiceImpl implements ComissaoService {

	@Autowired
	private ComissaoDAO comissaoDAO;

	@Override
	public List<Comissao> obtenerTodos() {
		return comissaoDAO.obtenerTodos();
	}

	@Override
	public Comissao obtenerID(Integer id) {
		return comissaoDAO.obtenerID(id);
	}

	@Override
	@Transactional
	public Comissao suscribir(Comissao entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return comissaoDAO.suscribir(entidad);
	}

	@Override
	@Transactional
	public Comissao actualizacion(Comissao entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());

		return comissaoDAO.actualizacion(entidad);
	}

	@Override
	@Transactional
	public void borrar(Comissao entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		comissaoDAO.actualizacion(entidad);
	}

	@Override
	public List<HistoricoComissao> buscaHistoricoComissao(Integer puntoVentaId, Integer empresaId, Date periodoInicio, Date periodoFim) {
		String competenciaInicial = DateUtil.getStringDate(periodoInicio, "MM/yyyy");
		String competenciaFinal = DateUtil.getStringDate(periodoFim, "MM/yyyy");
		
		Map<String, Object> parametros = new HashMap<String, Object>();
		parametros.put("competenciaInicial", competenciaInicial);
		parametros.put("competenciaFinal", competenciaFinal);
		parametros.put("empresaId", empresaId);
		
		if(puntoVentaId != null && puntoVentaId > -1) {
			parametros.put("puntoventaId", puntoVentaId);
		}
		
		return comissaoDAO.buscaHistoricoComissao(parametros);
	}

	@Override
	public List<PuntoVenta> buscaPuntoVentasEmpresaComComissaoParametrizada(Integer empresaId) {
		return comissaoDAO.buscaPuntoVentasEmpresaComComissaoParametrizada(empresaId);
	}

}
