package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.EmpresaCertificadoConfigDAO;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.EmpresaCertificadoConfig;
import com.rjconsultores.ventaboletos.enums.EnumTipoCertificado;
import com.rjconsultores.ventaboletos.service.EmpresaCertificadoConfigService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("empresaCertificadoConfigService")
public class EmpresaCertificadoConfigServiceImpl implements EmpresaCertificadoConfigService {

	@Autowired
	private EmpresaCertificadoConfigDAO empresaCertificadoConfigDAO;

	@Override
	public List<EmpresaCertificadoConfig> obtenerTodos() {
		return empresaCertificadoConfigDAO.obtenerTodos();
	}

	@Override
	public EmpresaCertificadoConfig obtenerID(Integer id) {
		return empresaCertificadoConfigDAO.obtenerID(id);
	}

	@Override
	@Transactional
	public EmpresaCertificadoConfig suscribir(EmpresaCertificadoConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return empresaCertificadoConfigDAO.suscribir(entidad);
	}

	@Override
	@Transactional
	public EmpresaCertificadoConfig actualizacion(EmpresaCertificadoConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return empresaCertificadoConfigDAO.actualizacion(entidad);
	}

	@Override
	@Transactional
	public void borrar(EmpresaCertificadoConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		empresaCertificadoConfigDAO.actualizacion(entidad);

	}

	@Override
	public EmpresaCertificadoConfig buscarPorEmpresa(Empresa empresa, EnumTipoCertificado enumTipoCertificado) {
		return empresaCertificadoConfigDAO.buscarPorEmpresa(empresa, enumTipoCertificado);
	}

}
