package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.EmpresaEmailConfigDAO;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.EmpresaEmailConfig;
import com.rjconsultores.ventaboletos.service.EmpresaEmailConfigService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("empresaEmailConfigService")
public class EmpresaEmailConfigServiceImpl implements EmpresaEmailConfigService {

	@Autowired
	private EmpresaEmailConfigDAO empresaEmailConfigDAO;

	@Override
	public List<EmpresaEmailConfig> obtenerTodos() {
		return empresaEmailConfigDAO.obtenerTodos();
	}

	@Override
	public EmpresaEmailConfig obtenerID(Integer id) {
		return empresaEmailConfigDAO.obtenerID(id);
	}

	@Override
	@Transactional
	public EmpresaEmailConfig suscribir(EmpresaEmailConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return empresaEmailConfigDAO.suscribir(entidad);
	}

	@Override
	@Transactional
	public EmpresaEmailConfig actualizacion(EmpresaEmailConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return empresaEmailConfigDAO.actualizacion(entidad);
	}

	@Override
	@Transactional
	public void borrar(EmpresaEmailConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		empresaEmailConfigDAO.actualizacion(entidad);

	}

	@Override
	public EmpresaEmailConfig buscarPorEmpresa(Empresa empresa) {
		return empresaEmailConfigDAO.buscarPorEmpresa(empresa);
	}

}
