package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;
import java.util.Optional;

import com.rjconsultores.ventaboletos.dao.EmpresaRecargaDAO;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.EmpresaRecargaConfig;
import com.rjconsultores.ventaboletos.service.EmpresaRecargaService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service("empresaRecargaService")
public class EmpresaRecargaServiceImpl implements EmpresaRecargaService {

	@Autowired
	private EmpresaRecargaDAO empresaRecargaService;

	@Override
	public List<EmpresaRecargaConfig> obtenerTodos() {
		return empresaRecargaService.obtenerTodos();
	}

	@Override
	public EmpresaRecargaConfig obtenerID(Integer id) {
		return empresaRecargaService.obtenerID(id);
	}

	@Override
	@Transactional
	public EmpresaRecargaConfig suscribir(EmpresaRecargaConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return empresaRecargaService.suscribir(entidad);
	}

	@Override
	@Transactional
	public EmpresaRecargaConfig actualizacion(EmpresaRecargaConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return empresaRecargaService.actualizacion(entidad);
	}

	@Override
	@Transactional
	public void borrar(EmpresaRecargaConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		empresaRecargaService.actualizacion(entidad);

	}

	@Override
	public Optional<EmpresaRecargaConfig> buscarPorEmpresa(Empresa empresa) {
		return empresaRecargaService.buscarPorEmpresa(empresa);
	}

}
